@@ -280,12 +280,13 @@ class SeqLinear(nn.Module):
     """
     def __init__(self, d_model, n_heads,
                  d_conv=4, d_state=None, d_inner=None,
-                 device="cpu"):
+                 chunk_size=64, device="cpu"):
         super().__init__()
         self.d_model = d_model
         self.d_state = d_state if d_state is not None else d_model
         self.d_inner = d_inner if d_inner is not None else d_model
         self.n_heads = n_heads
+        self.chunk_size = chunk_size
         self.device = device
         
         d_in_proj = 2 * self.d_state + self.d_inner + self.n_heads
@@ -299,6 +300,7 @@ class SeqLinear(nn.Module):
             padding=d_conv-1,
             device=device,
         )
+        self.A = nn.Parameter(torch.empty(n_heads, device=device))
         self.dt_bias = nn.Parameter(torch.empty(n_heads, device=device))
         self.out_proj = nn.Linear(self.d_inner, d_model, bias=False, device=device)
         
@@ -308,7 +310,7 @@ class SeqLinear(nn.Module):
         nn.init.xavier_uniform_(self.in_proj.weight)
         nn.init.xavier_uniform_(self.out_proj.weight)
         nn.init.xavier_uniform_(self.conv.weight)
-        nn.init.uniform_(self.dt_bias, -0.1, 0.1)
+        nn.init.uniform_(self.A, -0.1, 0.1)
         
         if self.conv.bias is not None:
             nn.init.constant_(self.conv.bias, 0.)
@@ -345,8 +347,8 @@ class SeqLinear(nn.Module):
                 activation=None,
             ).transpose(1, 2)
         else:
-            x_f = self.conv(x.transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
-            x_b = self.conv(x.flip(1).transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
+            x_f = self.conv(x.transpose(1, 2).contiguous()).transpose(1, 2)[:, :x.size(1)]
+            x_b = self.conv(x.flip(1).transpose(1, 2).contiguous()).transpose(1, 2)[:, :x.size(1)]
         
         C_f, B_f, x_f = torch.split(
             x_f,
@@ -367,24 +369,28 @@ class SeqLinear(nn.Module):
             dim=-1
         )
         
-        dt_f = F.softplus(dt + self.dt_bias)
         C_f = rearrange(C_f, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         B_f = rearrange(B_f, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         x_f = rearrange(x_f, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         
-        dt_b = F.softplus(dt.flip(1) + self.dt_bias)
         C_b = rearrange(C_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         B_b = rearrange(B_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         x_b = rearrange(x_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         
-        W_f = torch.cumsum(torch.einsum('bsh, bshd, bshe -> bshde', dt_f, B_f, x_f), dim=1)
-        W_b = torch.cumsum(torch.einsum('bsh, bshd, bshe -> bshde', dt_b, B_b, x_b), dim=1)
-        
-        norm_f = torch.cumsum(dt_f, dim=1).unsqueeze(-1)
-        norm_b = torch.cumsum(dt_b, dim=1).unsqueeze(-1)
-        
-        out_f = torch.einsum('bshd, bshde -> bshe', C_f, W_f) / norm_f
-        out_b = torch.einsum('bshd, bshde -> bshe', C_b, W_b) / norm_b
+        decay_f = F.silu(dt * self.A).transpose(1, 2).unsqueeze(-2).repeat(1, 1, src_len, 1)
+        decay_b = decay_f.flip(-1)
+        mask = torch.tril(torch.ones(src_len, src_len, dtype=torch.bool, device=self.device), diagonal=-1)
+        decay_f = decay_f.masked_fill(~mask, 0.0)
+        decay_b = decay_b.masked_fill(~mask, 0.0)
+        decay_f = -decay_f.flip(-1).cumsum(-1).flip(-1)
+        decay_b = -decay_b.flip(-1).cumsum(-1).flip(-1)
+        mask = torch.tril(torch.ones(src_len, src_len, dtype=torch.bool, device=self.device), diagonal=0)
+        decay_f = decay_f.masked_fill(~mask, float('-inf'))
+        decay_b = decay_b.masked_fill(~mask, float('-inf'))
+        decay_f = F.softmax(decay_f, dim=-1)
+        decay_b = F.softmax(decay_b, dim=-1)
+        out_f = torch.einsum('bthd, bhts, bshd, bshe -> bthe', C_f, decay_f, B_f, x_f)
+        out_b = torch.einsum('bthd, bhts, bshd, bshe -> bthe', C_b, decay_b, B_b, x_b)
         
         out = rearrange(out_f + out_b, 'b s h d -> b s (h d)')
         out = self.out_proj(out)
@@ -393,7 +399,7 @@ class SeqLinear(nn.Module):
     def forward(self, x, mode="causal"):
         if mode == "bidirectional": return self.bidirectional(x)
         bsz, src_len, d_model = x.size()
-        
+
         x = self.in_proj(x)
         
         x, dt = torch.split(
@@ -413,7 +419,7 @@ class SeqLinear(nn.Module):
                 activation=None,
             ).transpose(1, 2)
         else:
-            x = self.conv(x.transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
+            x = self.conv(x.transpose(1, 2).contiguous()).transpose(1, 2)[:, :x.size(1)]
         
         C, B, x = torch.split(
             x,
@@ -430,15 +436,18 @@ class SeqLinear(nn.Module):
         x = rearrange(x, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         
         if mode == "causal":
-            dt = F.softplus(dt + self.dt_bias)
-            norm = torch.cumsum(dt, dim=-1).unsqueeze(-1)
-            W = torch.cumsum(torch.einsum('bsh, bshd, bshe -> bshde', dt, C, B), dim=1)
-            out = torch.einsum('bshd, bshde -> bshe', x, W) / norm
+            decay = F.silu(dt * self.A).transpose(1, 2).unsqueeze(-2).repeat(1, 1, src_len, 1)
+            mask = torch.tril(torch.ones(src_len, src_len, dtype=torch.bool, device=self.device), diagonal=-1)
+            decay = decay.masked_fill(~mask, 0.0)
+            decay = -decay.flip(-1).cumsum(-1).flip(-1)
+            mask = torch.tril(torch.ones(src_len, src_len, dtype=torch.bool, device=self.device), diagonal=0)
+            decay = decay.masked_fill(~mask, float('-inf'))
+            decay = F.softmax(decay, dim=-1)
+            out = torch.einsum('bthd, bhts, bshd, bshe -> bthe', C, decay, B, x)
         elif mode == "noncausal":
-            dt = F.softplus(dt + self.dt_bias)
-            norm = torch.sum(dt, dim=1, keepdim=True).unsqueeze(-1)
-            out = torch.einsum('bshd, bthd, bthe -> bshe', x, C, B) / src_len
-        
+            decay = F.softmax(-F.silu(dt * self.A), dim=1)
+            out = torch.einsum('bthd, bsh, bshd, bshe -> bthe', C, decay, B, x)
+
         out = rearrange(out, 'b s h d -> b s (h d)')
         out = self.out_proj(out)
         return out
@@ -448,7 +457,7 @@ class SeqMLP(nn.Module):
                  n_layers=1, n_heads=1, dropout=0.0,
                  d_state=None, d_inner=None, d_conv=4,
                  pos_encoding=False, pos_encoding_max_len=None,
-                 use_embedding=True, weight_tying=False,
+                 use_embedding=True, weight_tying=False, chunk_size=64,
                  mode="causal", device="cpu"):
         super().__init__()
         self.emb_dim = emb_dim
@@ -480,6 +489,7 @@ class SeqMLP(nn.Module):
                         d_state=d_state,
                         d_inner=d_inner,
                         d_conv=d_conv,
+                        chunk_size=chunk_size,
                         device=device
                     ),
                     dropout = nn.Dropout(dropout),
@@ -505,8 +515,8 @@ class SeqMLP(nn.Module):
             x = x + self.abs_pos_encoding(pos)
         for layer in self.layers:
             x_norm = layer.norm(x)
-            y = layer.dropout(layer.mixer(x_norm, mode=self.mode))
-            x = x + y
+            mixer_out = layer.mixer(x_norm, mode=self.mode)
+            x = x + layer.dropout(mixer_out)
         x = self.norm_f(x)
         x = self.out_proj(x)
         return x
\ No newline at end of file
